﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace PripremaNoviZad
{
    public class BookManager
    {
        private IRepo repo;
        private IDictionary<string, Book> books;

        public event OnBookReturnDelegate OnBookReturn;

        public IDictionary<string, Book> Books {
            get
            {
                if (books == null)            
                {
                    Load();
                }

                return new Dictionary<string, Book>(books);
            }
        }

        public void CheckBooks()
        {
            try
            {
                foreach (Book book in books.Values.ToList())
                {
                    if (!book.Vraceno)
                    {
                        OnBookReturn?.Invoke(this, new OnBookReturnArgs { Returned = book.Vraceno });
                    }
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.ToString());
            }
        }

        public void AddBook(Book book)
        {
            try
            {
                if (books == null)
                {
                    throw new Exception("Dikšneri nije inicijaliziran");
                }
                if (books.ContainsKey(book.ISBN))
                {
                    throw new DuplicateRecordException();
                }
                books.Add(book.ISBN, book);
            }
            catch (DuplicateRecordException ex) {
                Console.WriteLine(ex.ToString());
            }
        }

        private void Load()
        {
            try
            {
                repo = RepoFactory.GetRepo();
                ISet<Book> bookSet = repo.Load();
                Process(bookSet);
            }
            catch (Exception ex) {
                Console.WriteLine(ex.ToString());
            }
        }

        private void Process(ISet<Book> bookSet)
        {
            books = new SortedDictionary<string, Book>();
            try
            {
                foreach (var book in bookSet)
                {
                    books.Add(book.ISBN, book);
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.ToString());
            }
        }
    }
}
